@@ -421,7 +421,7 @@ def command_install(lookup, packages, options):
         raise RosdepInternalError(e)
     except InstallFailed as e:
         print("ERROR: the following rosdeps failed to install", file=sys.stderr)
-        print('\n'.join(["  %s"%f for f in e.failures]), file=sys.stderr)
+        print('\n'.join(["  %s: %s"%(k, m) for k,m in e.failures]), file=sys.stderr)
         return 1
 
 def _compute_depdb_output(lookup, packages, options):
@@ -525,7 +525,7 @@ def command_resolve(args, options):
         installer = installer_context.get_installer(rule_installer)
         resolved = installer.resolve(rule)
         print("#%s"%(rule_installer))
-        print (" ".join(resolved))
+        print (" ".join([str(r) for r in resolved]))
 
     for error in invalid_key_errors:
         print("ERROR: no rosdep rule for %s"%(error), file=sys.stderr)        
